<?php

namespace Drupal\private_media;

use Drupal\file\FileInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Database\Driver\mysql\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class PrivateMediaService.
 */
class PrivateMediaService {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  /**
   * Constructs a new PrivateMediaService object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Connection $database, FileSystemInterface $fs) {
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
    $this->fileSystem = $fs;
  }

  /**
   * Determines if the file should be private or not.
   *
   * @todo Check moderation state also for TEA.
   */
  public function shouldBePrivate(EntityInterface $host_entity, string $field_name, FileInterface $file, string $op) {
    if ($op === 'uninstall' || $op === 'public') {
      return 'no';
    }

    $entity_type = $host_entity->getEntityTypeId();
    // Query revisions for all published statuses. Not using entity
    // query because we don't care about the actual media entities.

    $query = $this->database->select($entity_type . '_field_revision', 'mr')->fields('mr', ['status']);
    $query->join($entity_type . '_revision__' . $field_name, 'mf', 'mr.mid = mf.entity_id AND mr.vid = mf.revision_id');
    if (isset($host_entity->moderation_state)) {
      $query->join('content_moderation_state_field_revision', 'ms', 'ms.content_entity_type_id = \'media\' AND mr.mid = ms.content_entity_id AND mr.vid = ms.content_entity_revision_id');
      $query->fields('ms', ['moderation_state']);
    }
    $res = $query->condition('mf.' . $field_name . '_target_id', $file->id())->execute()->fetchAll();

    if (empty($res)) {
      // Not referenced as a file by any media entities.
      return 'unknown';
    }

    foreach ($res as $row) {
      // Look for that one published status to make it public.
      if (isset($row->status) && $row->status != 0) {
        return 'no';
      }

      // Look for that one moderation state to make it public.
      if (isset($row->moderation_state) && !in_array($row->moderation_state, ['draft', 'archived', 'recycle'])) {
        return 'no';
      }
    }

    return 'yes';
  }

  /**
   * Responds to an entity operation.
   *
   * @param Drupal\Core\Entity\EntityInterface $entity
   *   The entity being operated on.
   * @param string $op
   *   The operation name.
   */
  public function mediaEntityOperation(EntityInterface $entity, string $op) {
    foreach ($entity->getFields() as $field) {
      if ($field->getFieldDefinition()->getType() === 'file' && method_exists($field, 'referencedEntities')) {
        foreach ($field->referencedEntities() as $file) {
          $currentURI = $file->getFileUri();

          $idealURI = explode('://', $currentURI, 2);

          if (count($idealURI) < 2) {
            // URI is not able to be moved.
            continue;
          }

          switch ($this->shouldBePrivate($entity, $field->getName(), $file, $op)) {
            case "yes":
              $idealURI[0] = 'private';
              break;

            case "no":
              $idealURI[0] = 'public';
              break;

            default:
              continue;
          }

          $idealURI = implode('://', $idealURI);
          $this->moveFile($file, $idealURI);
        }
      }
    }
  }

  /**
   * Move the file from source uri to target uri.
   */
  private function moveFile(EntityInterface $fileEntity, string $newuri) {
    if (empty($fileEntity) || !method_exists($fileEntity, 'getFileUri') || !method_exists($fileEntity, 'setFileUri')) {
      return FALSE;
    }

    $olduri = $fileEntity->getFileUri();

    if ($olduri === $newuri) {
      return TRUE;
    }

    $oldexists = file_exists($olduri);
    $newexists = file_exists($newuri);

    if ($oldexists && !$newexists) {
      $newdir = pathinfo($newuri, PATHINFO_DIRNAME);
      if ($this->fileSystem->prepareDirectory($newdir, FileSystemInterface::CREATE_DIRECTORY)) {
        // Move file and update uri.
        if (!empty($this->fileSystem->move($olduri, $newuri, FileSystemInterface::EXISTS_ERROR))) {
          $fileEntity->setFileUri($newuri);
          $fileEntity->save();
          return TRUE;
        }
      }
    }

    if (!$oldexists && $newexists) {
      // File already there, update URI only.
      $fileEntity->setFileUri($newuri);
      $fileEntity->save();
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Wrapped by hook_help().
   *
   * @param mixed $route_name
   *   The route name.
   * @param Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match interface.
   */
  public function privateMediaHelp($route_name, RouteMatchInterface $route_match) {
    switch ($route_name) {
      // Main module help for the private_media module.
      case 'help.page.private_media':
        $output = '';
        $output .= '<h3>' . t('About') . '</h3>';
        $output .= '<p>' . t('Moves media files to the private file system when not referenced by a published revision') . '</p>';
        return $output;

      default:
    }
  }

}
